#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <richedit.h>
#include "ConvFunc.h"
#include "AkelEdit.h"
#include "AkelDLL.h"
#include "Resources\Resource.h"


//Include conversion functions
#define dec2hexW
#define hex2decW
#define xatoiA
#define xatoiW
#include "ConvFunc.h"

//Defines
#define STRID_DIRECTION  1
#define STRID_STR2HEX    2
#define STRID_HEX2STR    3
#define STRID_SETUP      4
#define STRID_ENCLOSE    5
#define STRID_PREFIX     6
#define STRID_SUFFIX     7
#define STRID_SINGLEBYTE 8
#define STRID_AUTO       9
#define STRID_DOUBLEBYTE 10
#define STRID_OK         11
#define STRID_CANCEL     12

#define CP_UNICODE_UCS2_LE  1200
#define CP_UNICODE_UCS2_BE  1201
#define CP_UNICODE_UTF7     65000
#define CP_UNICODE_UTF8     65001

#define STR_UNICODE_UCS2_LEA   "1200  (UTF-16 LE)"
#define STR_UNICODE_UCS2_LEW  L"1200  (UTF-16 LE)"
#define STR_UNICODE_UCS2_BEA   "1201  (UTF-16 BE)"
#define STR_UNICODE_UCS2_BEW  L"1201  (UTF-16 BE)"
#define STR_UNICODE_UTF8A      "65001 (UTF-8)"
#define STR_UNICODE_UTF8W     L"65001 (UTF-8)"
#define STR_UNICODE_UTF7A      "65000 (UTF-7)"
#define STR_UNICODE_UTF7W     L"65000 (UTF-7)"

#define CT_STR2HEX     1
#define CT_HEX2STR     2

#define AKDLL_GETHEX  (WM_USER + 1)
#define AKDLL_SETUP   (WM_USER + 2)

#define OF_WINDOWRECT     0x1
#define OF_SETTINGS       0x2

//DestroyDock type
#define DKT_UNLOAD         0
#define DKT_NOUNLOAD       1
#define DKT_ONMAINFINISH   2

//DIALOGRESIZE
#define DRS_SIZE  0x1 //Resize control
#define DRS_MOVE  0x2 //Move control
#define DRS_X     0x4 //X value
#define DRS_Y     0x8 //Y value

typedef struct {
  HWND *lpWnd;   //Control window
  DWORD dwType;  //See DRS_* defines
} DIALOGRESIZE;

//Functions prototypes
void CreateDock(HWND *hWndDock, DOCK **dkDock);
void DestroyDock(HWND hWndDock, DWORD dwType);
BOOL CALLBACK MainDlgProcA(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
BOOL CALLBACK MainDlgProcW(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
BOOL DialogResizeMessages(DIALOGRESIZE *drs, RECT *rcInit, RECT *rcCurrent, HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
BOOL GetWindowPos(HWND hWnd, HWND hWndOwner, RECT *rc);
BOOL CALLBACK SetupDlgProcA(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
BOOL CALLBACK SetupDlgProcW(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
LRESULT CALLBACK NewMainProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
LRESULT CALLBACK NewFrameProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
LRESULT CALLBACK EditParentMessages(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
void FillCodepageListboxA(HWND hWnd, int nCodePage);
void FillCodepageListboxW(HWND hWnd, int nCodePage);
void GetCodePageNameA(int nCodePage, char *szCodePage, int nLen);
void GetCodePageNameW(int nCodePage, wchar_t *wszCodePage, int nLen);
void GetHex(HWND hWndHexView, HWND hWndEdit);
int GetHexFromAnsiStringW(char *szInput, int nInputLen, wchar_t *wszPrefix, wchar_t *wszSuffix, wchar_t **wppOutput);
int GetHexFromUnicodeStringW(wchar_t *wszInput, int nInputLen, wchar_t *wszPrefix, wchar_t *wszSuffix, wchar_t **wppOutput);
int GetAnsiStringFromHexW(wchar_t *wszInput, int nInputLen, wchar_t *wszPrefix, wchar_t *wszSuffix, char **ppOutput);
int GetUnicodeStringFromHexW(wchar_t *wszInput, int nInputLen, wchar_t *wszPrefix, wchar_t *wszSuffix, wchar_t **wppOutput);
void FreeHexString(void *lpString);
void UpdateSelection();
void ReadOptionsA();
void ReadOptionsW();
void SaveOptionsA(DWORD dwFlags);
void SaveOptionsW(DWORD dwFlags);
char* GetLangStringA(LANGID wLangID, int nStringID);
wchar_t* GetLangStringW(LANGID wLangID, int nStringID);
void InitCommon(PLUGINDATA *pd);
void InitMain();
void UninitMain();

//GetProcAddress
BOOL (WINAPI *GetCPInfoExAPtr)(UINT, DWORD, LPCPINFOEXA);
BOOL (WINAPI *GetCPInfoExWPtr)(UINT, DWORD, LPCPINFOEXW);

//Global variables
char szPrefix[MAX_PATH]="<";
wchar_t wszPrefix[MAX_PATH]=L"<";
char szSuffix[MAX_PATH]=">";
wchar_t wszSuffix[MAX_PATH]=L">";
char szPluginName[MAX_PATH];
wchar_t wszPluginName[MAX_PATH];
HINSTANCE hInstanceDLL;
HWND hMainWnd;
HWND hWndEdit;
HMENU hPopupView;
HICON hMainIcon;
BOOL bOldWindows;
BOOL bAkelEdit;
BOOL bMDI;
LANGID wLangSystem;
BOOL bInitCommon=FALSE;
BOOL bInitMain=FALSE;
HWND hWndMainDlg=NULL;
RECT rcMainInitDialog={0};
RECT rcMainCurrentDialog={0};
RECT rcDockRect={0};
int nDockSide=DKS_BOTTOM;
DOCK *dkMainDlg=NULL;
BOOL bOnMainStart=FALSE;
BOOL bDockWaitResize=FALSE;
BOOL bMainDialogRectSave=FALSE;
int nDirection=CT_STR2HEX;
int nAnsiCP=-1;
int nUnicodeCP=CP_UNICODE_UCS2_LE;
WNDPROCDATA *NewMainProcData=NULL;
WNDPROCDATA *NewFrameProcData=NULL;


//Identification
extern "C" void __declspec(dllexport) DllAkelPadID(PLUGINVERSION *pv)
{
  pv->dwAkelDllVersion=AKELDLL;
  pv->dwExeMinVersion3x=MAKE_IDENTIFIER(3, 7, 4, 0);
  pv->dwExeMinVersion4x=MAKE_IDENTIFIER(4, 4, 0, 0);
  pv->pPluginName="HexSel";
}

//Plugin extern functions
extern "C" void __declspec(dllexport) Main(PLUGINDATA *pd)
{
  if (pd->lpbAutoLoad)
  {
    *pd->lpbAutoLoad=TRUE;
    return;
  }

  if (!bInitCommon) InitCommon(pd);

  //Is plugin already loaded?
  if (bInitMain)
  {
    if (hWndMainDlg) DestroyDock(hWndMainDlg, DKT_UNLOAD);

    //Stay in memory, but show as non-active
    pd->nUnload=UD_NONUNLOAD_NONACTIVE;
  }
  else
  {
    InitMain();

    bDockWaitResize=pd->bOnStart;
    CreateDock(&hWndMainDlg, &dkMainDlg);

    //Stay in memory, and show as active
    pd->nUnload=UD_NONUNLOAD_ACTIVE;
  }
}

void CreateDock(HWND *hWndDock, DOCK **dkDock)
{
  DOCK dk={0};

  *hWndDock=NULL;
  dk.dwFlags=DKF_DRAGDROP;
  dk.nSide=nDockSide;
  dk.rcSize=rcDockRect;

  if (*dkDock=(DOCK *)SendMessage(hMainWnd, AKD_DOCK, DK_ADD, (LPARAM)&dk))
  {
    if (bOldWindows)
      *hWndDock=CreateDialogA(hInstanceDLL, MAKEINTRESOURCEA(IDD_HEXSEL), hMainWnd, (DLGPROC)MainDlgProcA);
    else
      *hWndDock=CreateDialogW(hInstanceDLL, MAKEINTRESOURCEW(IDD_HEXSEL), hMainWnd, (DLGPROC)MainDlgProcW);

    if (*hWndDock)
    {
      if (!(*dkDock)->rcSize.right || !(*dkDock)->rcSize.bottom)
      {
        GetWindowRect(*hWndDock, &(*dkDock)->rcSize);
        (*dkDock)->rcSize.right-=(*dkDock)->rcSize.left;
        (*dkDock)->rcSize.bottom-=(*dkDock)->rcSize.top;
        (*dkDock)->rcSize.left=0;
        (*dkDock)->rcSize.top=0;
      }
      (*dkDock)->hWnd=*hWndDock;
      SendMessage(hMainWnd, AKD_DOCK, DK_SUBCLASS, (LPARAM)*dkDock);

      if (!bDockWaitResize)
      {
        SendMessage(hMainWnd, AKD_RESIZE, 0, 0);
        ShowWindow(*hWndDock, SW_SHOW);
      }
    }
  }
}

void DestroyDock(HWND hWndDock, DWORD dwType)
{
  SendMessage(hWndDock, WM_COMMAND, IDC_CLOSE, dwType);
}

BOOL CALLBACK MainDlgProcA(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
  static HWND hWndHexView;
  static HWND hWndTitleText;
  static HWND hWndTitleClose;
  static HMENU hMenuLabel;
  static DIALOGRESIZE drs[]={{&hWndTitleText,    DRS_SIZE|DRS_X},
                             {&hWndTitleClose,   DRS_MOVE|DRS_X},
                             {&hWndHexView,      DRS_SIZE|DRS_X|DRS_Y},
                             {0, 0}};

  if (uMsg == WM_INITDIALOG)
  {
    hWndHexView=GetDlgItem(hDlg, IDC_HEXVIEW);
    hWndTitleText=GetDlgItem(hDlg, IDC_TITLETEXT);
    hWndTitleClose=GetDlgItem(hDlg, IDC_CLOSE);

    SendMessage(hWndHexView, EM_SETUNDOLIMIT, 0, 0);
    SendMessage(hWndHexView, EM_SHOWSCROLLBAR, SB_HORZ, FALSE);
    if (bAkelEdit)
      SendMessage(hWndHexView, AEM_SETWORDWRAP, AEWW_WORD, 0);
    else
      SendMessage(hWndHexView, EM_SETTARGETDEVICE, (WPARAM)NULL, 0);

    hMenuLabel=CreatePopupMenu();
    AppendMenuA(hMenuLabel, MF_STRING, IDM_SETUP, GetLangStringA(wLangSystem, STRID_SETUP));

    SetWindowTextA(hWndTitleText, szPluginName);
    SendMessage(hMainWnd, AKD_SETCLOSEBUTTON, (WPARAM)hWndTitleClose, (LPARAM)NULL);

    if (!bOnMainStart) PostMessage(hDlg, AKDLL_GETHEX, 0, 0);
  }
  else if (uMsg == AKDLL_GETHEX)
  {
    GetHex(hWndHexView, (HWND)wParam);
  }
  else if (uMsg == AKDLL_SETUP)
  {
    DialogBoxA(hInstanceDLL, MAKEINTRESOURCEA(IDD_SETUP), hDlg, (DLGPROC)SetupDlgProcA);
  }
  else if (uMsg == WM_LBUTTONDBLCLK)
  {
    POINT pt;
    RECT rc;

    GetCursorPos(&pt);
    GetWindowRect(hWndTitleText, &rc);

    if (PtInRect(&rc, pt))
    {
      PostMessage(hDlg, AKDLL_SETUP, 0, 0);
    }
  }
  else if (uMsg == WM_CONTEXTMENU)
  {
    if ((HWND)wParam == hDlg)
    {
      POINT pt;
      RECT rc;

      GetCursorPos(&pt);
      GetWindowRect(hWndTitleText, &rc);

      if (PtInRect(&rc, pt))
      {
        int nCmd;

        nCmd=TrackPopupMenu(hMenuLabel, TPM_RETURNCMD|TPM_NONOTIFY|TPM_LEFTBUTTON|TPM_RIGHTBUTTON, pt.x, pt.y, 0, hWndTitleText, NULL);

        if (nCmd == IDM_SETUP)
          PostMessage(hDlg, AKDLL_SETUP, 0, 0);
      }
    }
    else if ((HWND)wParam == hWndHexView)
    {
      RECT rcRect;
      POINT pt;
      CHARRANGE cr;
      int nCmd;

      if (lParam == -1)
      {
        GetWindowRect((HWND)wParam, &rcRect);
        pt.x=rcRect.left;
        pt.y=rcRect.top;
      }
      else
      {
        GetCursorPos(&pt);
      }
      SendMessage((HWND)wParam, EM_EXGETSEL, 0, (LPARAM)&cr);
      EnableMenuItem(hPopupView, IDM_EDIT_COPY, (cr.cpMin != cr.cpMax)?MF_ENABLED:MF_GRAYED);
      nCmd=TrackPopupMenu(hPopupView, TPM_RETURNCMD|TPM_NONOTIFY|TPM_LEFTBUTTON|TPM_RIGHTBUTTON, pt.x, pt.y, 0, hDlg, NULL);

      if (nCmd == IDM_EDIT_COPY)
        SendMessage((HWND)wParam, WM_COPY, 0, 0);
      else if (nCmd == IDM_EDIT_SELECTALL)
        SendMessage((HWND)wParam, EM_SETSEL, 0, -1);
    }
  }
  else if (uMsg == WM_COMMAND)
  {
    if (LOWORD(wParam) == IDC_CLOSE ||
        LOWORD(wParam) == IDCANCEL)
    {
      if (bMainDialogRectSave || dkMainDlg->nSide != nDockSide)
        SaveOptionsA(OF_WINDOWRECT);

      if (lParam != DKT_NOUNLOAD)
      {
        UninitMain();
      }
      SendMessage(hMainWnd, AKD_DOCK, DK_DELETE, (LPARAM)dkMainDlg);
      dkMainDlg=NULL;

      DestroyMenu(hMenuLabel);
      DestroyWindow(hWndMainDlg);
      hWndMainDlg=NULL;

      if (lParam != DKT_ONMAINFINISH)
      {
        SendMessage(hMainWnd, AKD_RESIZE, 0, 0);
        SetFocus(hMainWnd);
        if (lParam != DKT_NOUNLOAD)
          PostMessage(hMainWnd, AKD_DLLUNLOAD, (WPARAM)hInstanceDLL, (LPARAM)NULL);
      }
    }
  }
  else if (uMsg == WM_CLOSE)
  {
    PostMessage(hDlg, WM_COMMAND, IDCANCEL, 0);
    return TRUE;
  }
  if (DialogResizeMessages(&drs[0], &rcMainInitDialog, &rcMainCurrentDialog, hDlg, uMsg, wParam, lParam))
  {
    if (IsWindowVisible(hDlg)) bMainDialogRectSave=TRUE;
    if (dkMainDlg) GetWindowPos(hWndTitleText, hDlg, &dkMainDlg->rcDragDrop);
  }

  return FALSE;
}

BOOL CALLBACK MainDlgProcW(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
  static HWND hWndHexView;
  static HWND hWndTitleText;
  static HWND hWndTitleClose;
  static HMENU hMenuLabel;
  static DIALOGRESIZE drs[]={{&hWndTitleText,    DRS_SIZE|DRS_X},
                             {&hWndTitleClose,   DRS_MOVE|DRS_X},
                             {&hWndHexView,      DRS_SIZE|DRS_X|DRS_Y},
                             {0, 0}};

  if (uMsg == WM_INITDIALOG)
  {
    hWndHexView=GetDlgItem(hDlg, IDC_HEXVIEW);
    hWndTitleText=GetDlgItem(hDlg, IDC_TITLETEXT);
    hWndTitleClose=GetDlgItem(hDlg, IDC_CLOSE);

    SendMessage(hWndHexView, EM_SETUNDOLIMIT, 0, 0);
    SendMessage(hWndHexView, EM_SHOWSCROLLBAR, SB_HORZ, FALSE);
    if (bAkelEdit)
      SendMessage(hWndHexView, AEM_SETWORDWRAP, AEWW_WORD, 0);
    else
      SendMessage(hWndHexView, EM_SETTARGETDEVICE, (WPARAM)NULL, 0);

    hMenuLabel=CreatePopupMenu();
    AppendMenuW(hMenuLabel, MF_STRING, IDM_SETUP, GetLangStringW(wLangSystem, STRID_SETUP));

    SetWindowTextW(hWndTitleText, wszPluginName);
    SendMessage(hMainWnd, AKD_SETCLOSEBUTTON, (WPARAM)hWndTitleClose, (LPARAM)NULL);

    if (!bOnMainStart) PostMessage(hDlg, AKDLL_GETHEX, 0, 0);
  }
  else if (uMsg == AKDLL_GETHEX)
  {
    GetHex(hWndHexView, (HWND)wParam);
  }
  else if (uMsg == AKDLL_SETUP)
  {
    DialogBoxW(hInstanceDLL, MAKEINTRESOURCEW(IDD_SETUP), hDlg, (DLGPROC)SetupDlgProcW);
  }
  else if (uMsg == WM_LBUTTONDBLCLK)
  {
    POINT pt;
    RECT rc;

    GetCursorPos(&pt);
    GetWindowRect(hWndTitleText, &rc);

    if (PtInRect(&rc, pt))
    {
      PostMessage(hDlg, AKDLL_SETUP, 0, 0);
    }
  }
  else if (uMsg == WM_CONTEXTMENU)
  {
    if ((HWND)wParam == hDlg)
    {
      POINT pt;
      RECT rc;

      GetCursorPos(&pt);
      GetWindowRect(hWndTitleText, &rc);

      if (PtInRect(&rc, pt))
      {
        int nCmd;

        nCmd=TrackPopupMenu(hMenuLabel, TPM_RETURNCMD|TPM_NONOTIFY|TPM_LEFTBUTTON|TPM_RIGHTBUTTON, pt.x, pt.y, 0, hWndTitleText, NULL);

        if (nCmd == IDM_SETUP)
          PostMessage(hDlg, AKDLL_SETUP, 0, 0);
      }
    }
    else if ((HWND)wParam == hWndHexView)
    {
      RECT rcRect;
      POINT pt;
      CHARRANGE cr;
      int nCmd;

      if (lParam == -1)
      {
        GetWindowRect((HWND)wParam, &rcRect);
        pt.x=rcRect.left;
        pt.y=rcRect.top;
      }
      else
      {
        GetCursorPos(&pt);
      }
      SendMessage((HWND)wParam, EM_EXGETSEL, 0, (LPARAM)&cr);
      EnableMenuItem(hPopupView, IDM_EDIT_COPY, (cr.cpMin != cr.cpMax)?MF_ENABLED:MF_GRAYED);
      nCmd=TrackPopupMenu(hPopupView, TPM_RETURNCMD|TPM_NONOTIFY|TPM_LEFTBUTTON|TPM_RIGHTBUTTON, pt.x, pt.y, 0, hDlg, NULL);

      if (nCmd == IDM_EDIT_COPY)
        SendMessage((HWND)wParam, WM_COPY, 0, 0);
      else if (nCmd == IDM_EDIT_SELECTALL)
        SendMessage((HWND)wParam, EM_SETSEL, 0, -1);
    }
  }
  else if (uMsg == WM_COMMAND)
  {
    if (LOWORD(wParam) == IDC_CLOSE ||
        LOWORD(wParam) == IDCANCEL)
    {
      if (bMainDialogRectSave || dkMainDlg->nSide != nDockSide)
        SaveOptionsW(OF_WINDOWRECT);

      if (lParam != DKT_NOUNLOAD)
      {
        UninitMain();
      }
      SendMessage(hMainWnd, AKD_DOCK, DK_DELETE, (LPARAM)dkMainDlg);
      dkMainDlg=NULL;

      DestroyMenu(hMenuLabel);
      DestroyWindow(hWndMainDlg);
      hWndMainDlg=NULL;

      if (lParam != DKT_ONMAINFINISH)
      {
        SendMessage(hMainWnd, AKD_RESIZE, 0, 0);
        SetFocus(hMainWnd);
        if (lParam != DKT_NOUNLOAD)
          PostMessage(hMainWnd, AKD_DLLUNLOAD, (WPARAM)hInstanceDLL, (LPARAM)NULL);
      }
    }
  }
  else if (uMsg == WM_CLOSE)
  {
    PostMessage(hDlg, WM_COMMAND, IDCANCEL, 0);
    return TRUE;
  }
  if (DialogResizeMessages(&drs[0], &rcMainInitDialog, &rcMainCurrentDialog, hDlg, uMsg, wParam, lParam))
  {
    if (IsWindowVisible(hDlg)) bMainDialogRectSave=TRUE;
    if (dkMainDlg) GetWindowPos(hWndTitleText, hDlg, &dkMainDlg->rcDragDrop);
  }

  return FALSE;
}

BOOL DialogResizeMessages(DIALOGRESIZE *drs, RECT *rcInit, RECT *rcCurrent, HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
  if (uMsg == WM_INITDIALOG)
  {
    RECT rcTemplate;

    GetWindowPos(hDlg, NULL, rcInit);
    rcTemplate=*rcCurrent;
    *rcCurrent=*rcInit;

    if (rcTemplate.right && rcTemplate.bottom)
    {
      rcTemplate.left=rcInit->left + (rcInit->right - rcTemplate.right) / 2;
      rcTemplate.top=rcInit->top + (rcInit->bottom - rcTemplate.bottom) / 2;
      SetWindowPos(hDlg, 0, rcTemplate.left, rcTemplate.top, rcTemplate.right, rcTemplate.bottom, SWP_NOZORDER);
    }
  }
  else if (uMsg == WM_GETMINMAXINFO)
  {
    //MINMAXINFO *mmi=(MINMAXINFO *)lParam;
    //
    //mmi->ptMinTrackSize.x=rcInit->right;
    //mmi->ptMinTrackSize.y=rcInit->bottom;
  }
  else if (uMsg == WM_SIZE)
  {
    if (lParam)
    {
      RECT rcTemplate;
      RECT rcControl;
      DWORD dwFlags;
      POINT pt;
      int i;

      GetWindowPos(hDlg, NULL, &rcTemplate);
      pt.x=rcTemplate.right - rcCurrent->right;
      pt.y=rcTemplate.bottom - rcCurrent->bottom;
      *rcCurrent=rcTemplate;

      for (i=0; drs[i].lpWnd; ++i)
      {
        if (*drs[i].lpWnd)
        {
          dwFlags=0;
          if (drs[i].dwType & DRS_SIZE)
            dwFlags|=SWP_NOMOVE;
          else if (drs[i].dwType & DRS_MOVE)
            dwFlags|=SWP_NOSIZE;
          GetWindowPos(*drs[i].lpWnd, hDlg, &rcControl);
          SetWindowPos(*drs[i].lpWnd, 0, rcControl.left + ((drs[i].dwType & DRS_X)?pt.x:0), rcControl.top + ((drs[i].dwType & DRS_Y)?pt.y:0), rcControl.right + ((drs[i].dwType & DRS_X)?pt.x:0), rcControl.bottom + ((drs[i].dwType & DRS_Y)?pt.y:0), dwFlags|SWP_NOZORDER);
        }
      }
      InvalidateRect(hDlg, NULL, TRUE);
      return TRUE;
    }
  }
  else if (uMsg == WM_PAINT)
  {
    //PAINTSTRUCT ps;
    //RECT rcGrip;
    //HDC hDC;
    //
    //if (hDC=BeginPaint(hDlg, &ps))
    //{
    //  GetClientRect(hDlg, &rcGrip);
    //  rcGrip.left=rcGrip.right - GetSystemMetrics(SM_CXVSCROLL);
    //  rcGrip.top=rcGrip.bottom - GetSystemMetrics(SM_CYVSCROLL);
    //  DrawFrameControl(hDC, &rcGrip, DFC_SCROLL, DFCS_SCROLLSIZEGRIP);
    //  EndPaint(hDlg, &ps);
    //}
  }
  return FALSE;
}

BOOL GetWindowPos(HWND hWnd, HWND hWndOwner, RECT *rc)
{
  if (GetWindowRect(hWnd, rc))
  {
    rc->right-=rc->left;
    rc->bottom-=rc->top;

    if (hWndOwner)
    {
      if (!ScreenToClient(hWndOwner, (POINT *)&rc->left))
        return FALSE;
    }
    return TRUE;
  }
  return FALSE;
}

BOOL CALLBACK SetupDlgProcA(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
  static HWND hWndStr2Hex;
  static HWND hWndHex2Str;
  static HWND hWndPrefix;
  static HWND hWndSuffix;
  static HWND hWndAuto;
  static HWND hWndManual;
  static HWND hWndList;
  static HWND hWndUtf16LE;
  static HWND hWndUtf16BE;
  BOOL bState;

  if (uMsg == WM_INITDIALOG)
  {
    SendMessage(hDlg, WM_SETICON, (WPARAM)ICON_BIG, (LPARAM)hMainIcon);
    hWndStr2Hex=GetDlgItem(hDlg, IDC_SETUP_DIRECTION_STR2HEX);
    hWndHex2Str=GetDlgItem(hDlg, IDC_SETUP_DIRECTION_HEX2STR);
    hWndPrefix=GetDlgItem(hDlg, IDC_SETUP_PREFIX);
    hWndSuffix=GetDlgItem(hDlg, IDC_SETUP_SUFFIX);
    hWndAuto=GetDlgItem(hDlg, IDC_SETUP_SINGLEBYTE_AUTO);
    hWndManual=GetDlgItem(hDlg, IDC_SETUP_SINGLEBYTE_MANUAL);
    hWndList=GetDlgItem(hDlg, IDC_SETUP_SINGLEBYTE_LIST);
    hWndUtf16LE=GetDlgItem(hDlg, IDC_SETUP_DOUBLEBYTE_UTF16LE);
    hWndUtf16BE=GetDlgItem(hDlg, IDC_SETUP_DOUBLEBYTE_UTF16BE);

    SetDlgItemTextA(hDlg, IDC_SETUP_DIRECTION_LABEL, GetLangStringA(wLangSystem, STRID_DIRECTION));
    SetDlgItemTextA(hDlg, IDC_SETUP_DIRECTION_STR2HEX, GetLangStringA(wLangSystem, STRID_STR2HEX));
    SetDlgItemTextA(hDlg, IDC_SETUP_DIRECTION_HEX2STR, GetLangStringA(wLangSystem, STRID_HEX2STR));
    SetDlgItemTextA(hDlg, IDC_SETUP_ENCLOSE_LABEL, GetLangStringA(wLangSystem, STRID_ENCLOSE));
    SetDlgItemTextA(hDlg, IDC_SETUP_PREFIX_LABEL, GetLangStringA(wLangSystem, STRID_PREFIX));
    SetDlgItemTextA(hDlg, IDC_SETUP_SUFFIX_LABEL, GetLangStringA(wLangSystem, STRID_SUFFIX));
    SetDlgItemTextA(hDlg, IDC_SETUP_SINGLEBYTE_LABEL, GetLangStringA(wLangSystem, STRID_SINGLEBYTE));
    SetDlgItemTextA(hDlg, IDC_SETUP_SINGLEBYTE_AUTO, GetLangStringA(wLangSystem, STRID_AUTO));
    SetDlgItemTextA(hDlg, IDC_SETUP_DOUBLEBYTE_LABEL, GetLangStringA(wLangSystem, STRID_DOUBLEBYTE));
    SetDlgItemTextA(hDlg, IDOK, GetLangStringA(wLangSystem, STRID_OK));
    SetDlgItemTextA(hDlg, IDCANCEL, GetLangStringA(wLangSystem, STRID_CANCEL));

    if (nDirection == CT_STR2HEX)
      SendMessage(hWndStr2Hex, BM_SETCHECK, BST_CHECKED, 0);
    else if (nDirection == CT_HEX2STR)
      SendMessage(hWndHex2Str, BM_SETCHECK, BST_CHECKED, 0);

    SendMessage(hWndPrefix, EM_LIMITTEXT, MAX_PATH, 0);
    SendMessage(hWndSuffix, EM_LIMITTEXT, MAX_PATH, 0);
    SetWindowTextA(hWndPrefix, szPrefix);
    SetWindowTextA(hWndSuffix, szSuffix);

    FillCodepageListboxA(hWndList, nAnsiCP);
    if (nAnsiCP == -1)
    {
      SendMessage(hWndAuto, BM_SETCHECK, BST_CHECKED, 0);
      EnableWindow(hWndList, FALSE);
    }
    else SendMessage(hWndManual, BM_SETCHECK, BST_CHECKED, 0);

    if (nUnicodeCP == CP_UNICODE_UCS2_LE)
      SendMessage(hWndUtf16LE, BM_SETCHECK, BST_CHECKED, 0);
    else
      SendMessage(hWndUtf16BE, BM_SETCHECK, BST_CHECKED, 0);
  }
  else if (uMsg == WM_COMMAND)
  {
    if (LOWORD(wParam) == IDC_SETUP_SINGLEBYTE_AUTO ||
        LOWORD(wParam) == IDC_SETUP_SINGLEBYTE_MANUAL)
    {
      bState=SendMessage(hWndManual, BM_GETCHECK, 0, 0);
      EnableWindow(hWndList, bState);
    }
    else if (LOWORD(wParam) == IDOK)
    {
      if (SendMessage(hWndStr2Hex, BM_GETCHECK, 0, 0) == BST_CHECKED)
        nDirection=CT_STR2HEX;
      else if (SendMessage(hWndHex2Str, BM_GETCHECK, 0, 0) == BST_CHECKED)
        nDirection=CT_HEX2STR;

      GetWindowTextA(hWndPrefix, szPrefix, MAX_PATH);
      GetWindowTextA(hWndSuffix, szSuffix, MAX_PATH);
      MultiByteToWideChar(CP_ACP, 0, szPrefix, -1, wszPrefix, MAX_PATH);
      MultiByteToWideChar(CP_ACP, 0, szSuffix, -1, wszSuffix, MAX_PATH);

      if (SendMessage(hWndAuto, BM_GETCHECK, 0, 0) == BST_CHECKED)
        nAnsiCP=-1;
      else
        nAnsiCP=GetDlgItemInt(hDlg, IDC_SETUP_SINGLEBYTE_LIST, NULL, FALSE);

      if (SendMessage(hWndUtf16LE, BM_GETCHECK, 0, 0) == BST_CHECKED)
        nUnicodeCP=CP_UNICODE_UCS2_LE;
      else if (SendMessage(hWndUtf16BE, BM_GETCHECK, 0, 0) == BST_CHECKED)
        nUnicodeCP=CP_UNICODE_UCS2_BE;

      SaveOptionsA(OF_SETTINGS);
      UpdateSelection();

      EndDialog(hDlg, 0);
      return TRUE;
    }
    else if (LOWORD(wParam) == IDCANCEL)
    {
      EndDialog(hDlg, 0);
      return TRUE;
    }
  }
  else if (uMsg == WM_CLOSE)
  {
    PostMessage(hDlg, WM_COMMAND, IDCANCEL, 0);
    return TRUE;
  }
  return FALSE;
}

BOOL CALLBACK SetupDlgProcW(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
  static HWND hWndStr2Hex;
  static HWND hWndHex2Str;
  static HWND hWndPrefix;
  static HWND hWndSuffix;
  static HWND hWndAuto;
  static HWND hWndManual;
  static HWND hWndList;
  static HWND hWndUtf16LE;
  static HWND hWndUtf16BE;
  BOOL bState;

  if (uMsg == WM_INITDIALOG)
  {
    SendMessage(hDlg, WM_SETICON, (WPARAM)ICON_BIG, (LPARAM)hMainIcon);
    hWndStr2Hex=GetDlgItem(hDlg, IDC_SETUP_DIRECTION_STR2HEX);
    hWndHex2Str=GetDlgItem(hDlg, IDC_SETUP_DIRECTION_HEX2STR);
    hWndPrefix=GetDlgItem(hDlg, IDC_SETUP_PREFIX);
    hWndSuffix=GetDlgItem(hDlg, IDC_SETUP_SUFFIX);
    hWndAuto=GetDlgItem(hDlg, IDC_SETUP_SINGLEBYTE_AUTO);
    hWndManual=GetDlgItem(hDlg, IDC_SETUP_SINGLEBYTE_MANUAL);
    hWndList=GetDlgItem(hDlg, IDC_SETUP_SINGLEBYTE_LIST);
    hWndUtf16LE=GetDlgItem(hDlg, IDC_SETUP_DOUBLEBYTE_UTF16LE);
    hWndUtf16BE=GetDlgItem(hDlg, IDC_SETUP_DOUBLEBYTE_UTF16BE);

    SetDlgItemTextW(hDlg, IDC_SETUP_DIRECTION_LABEL, GetLangStringW(wLangSystem, STRID_DIRECTION));
    SetDlgItemTextW(hDlg, IDC_SETUP_DIRECTION_STR2HEX, GetLangStringW(wLangSystem, STRID_STR2HEX));
    SetDlgItemTextW(hDlg, IDC_SETUP_DIRECTION_HEX2STR, GetLangStringW(wLangSystem, STRID_HEX2STR));
    SetDlgItemTextW(hDlg, IDC_SETUP_ENCLOSE_LABEL, GetLangStringW(wLangSystem, STRID_ENCLOSE));
    SetDlgItemTextW(hDlg, IDC_SETUP_PREFIX_LABEL, GetLangStringW(wLangSystem, STRID_PREFIX));
    SetDlgItemTextW(hDlg, IDC_SETUP_SUFFIX_LABEL, GetLangStringW(wLangSystem, STRID_SUFFIX));
    SetDlgItemTextW(hDlg, IDC_SETUP_SINGLEBYTE_LABEL, GetLangStringW(wLangSystem, STRID_SINGLEBYTE));
    SetDlgItemTextW(hDlg, IDC_SETUP_SINGLEBYTE_AUTO, GetLangStringW(wLangSystem, STRID_AUTO));
    SetDlgItemTextW(hDlg, IDC_SETUP_DOUBLEBYTE_LABEL, GetLangStringW(wLangSystem, STRID_DOUBLEBYTE));
    SetDlgItemTextW(hDlg, IDOK, GetLangStringW(wLangSystem, STRID_OK));
    SetDlgItemTextW(hDlg, IDCANCEL, GetLangStringW(wLangSystem, STRID_CANCEL));

    if (nDirection == CT_STR2HEX)
      SendMessage(hWndStr2Hex, BM_SETCHECK, BST_CHECKED, 0);
    else if (nDirection == CT_HEX2STR)
      SendMessage(hWndHex2Str, BM_SETCHECK, BST_CHECKED, 0);

    SendMessage(hWndPrefix, EM_LIMITTEXT, MAX_PATH, 0);
    SendMessage(hWndSuffix, EM_LIMITTEXT, MAX_PATH, 0);
    SetWindowTextW(hWndPrefix, wszPrefix);
    SetWindowTextW(hWndSuffix, wszSuffix);

    FillCodepageListboxW(hWndList, nAnsiCP);
    if (nAnsiCP == -1)
    {
      SendMessage(hWndAuto, BM_SETCHECK, BST_CHECKED, 0);
      EnableWindow(hWndList, FALSE);
    }
    else SendMessage(hWndManual, BM_SETCHECK, BST_CHECKED, 0);

    if (nUnicodeCP == CP_UNICODE_UCS2_LE)
      SendMessage(hWndUtf16LE, BM_SETCHECK, BST_CHECKED, 0);
    else
      SendMessage(hWndUtf16BE, BM_SETCHECK, BST_CHECKED, 0);
  }
  else if (uMsg == WM_COMMAND)
  {
    if (LOWORD(wParam) == IDC_SETUP_SINGLEBYTE_AUTO ||
        LOWORD(wParam) == IDC_SETUP_SINGLEBYTE_MANUAL)
    {
      bState=SendMessage(hWndManual, BM_GETCHECK, 0, 0);
      EnableWindow(hWndList, bState);
    }
    else if (LOWORD(wParam) == IDOK)
    {
      if (SendMessage(hWndStr2Hex, BM_GETCHECK, 0, 0) == BST_CHECKED)
        nDirection=CT_STR2HEX;
      else if (SendMessage(hWndHex2Str, BM_GETCHECK, 0, 0) == BST_CHECKED)
        nDirection=CT_HEX2STR;

      GetWindowTextW(hWndPrefix, wszPrefix, MAX_PATH);
      GetWindowTextW(hWndSuffix, wszSuffix, MAX_PATH);

      if (SendMessage(hWndAuto, BM_GETCHECK, 0, 0) == BST_CHECKED)
        nAnsiCP=-1;
      else
        nAnsiCP=GetDlgItemInt(hDlg, IDC_SETUP_SINGLEBYTE_LIST, NULL, FALSE);

      if (SendMessage(hWndUtf16LE, BM_GETCHECK, 0, 0) == BST_CHECKED)
        nUnicodeCP=CP_UNICODE_UCS2_LE;
      else if (SendMessage(hWndUtf16BE, BM_GETCHECK, 0, 0) == BST_CHECKED)
        nUnicodeCP=CP_UNICODE_UCS2_BE;

      SaveOptionsW(OF_SETTINGS);
      UpdateSelection();

      EndDialog(hDlg, 0);
      return TRUE;
    }
    else if (LOWORD(wParam) == IDCANCEL)
    {
      EndDialog(hDlg, 0);
      return TRUE;
    }
  }
  else if (uMsg == WM_CLOSE)
  {
    PostMessage(hDlg, WM_COMMAND, IDCANCEL, 0);
    return TRUE;
  }
  return FALSE;
}

LRESULT CALLBACK NewMainProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
  if (uMsg == AKDN_MAIN_ONSTART_IDLE)
  {
    if (bOnMainStart) PostMessage(hWndMainDlg, AKDLL_GETHEX, 0, 0);
  }
  else if (uMsg == AKDN_FRAME_NOWINDOWS)
  {
    SendMessage(hWndMainDlg, AKDLL_GETHEX, 0, 0);
  }
  else if (uMsg == AKDN_SIZE)
  {
    if (bDockWaitResize)
    {
      bDockWaitResize=FALSE;
      ShowWindow(hWndMainDlg, SW_SHOW);
    }
  }
  else if (uMsg == AKDN_MAIN_ONFINISH)
  {
    NewMainProcData->NextProc(hWnd, uMsg, wParam, lParam);

    if (hWndMainDlg)
    {
      DestroyDock(hWndMainDlg, DKT_ONMAINFINISH);
    }
    return FALSE;
  }

  //Edit parent
  {
   LRESULT lResult;

   if (lResult=EditParentMessages(hWnd, uMsg, wParam, lParam)) return lResult;
  }

  //Call next procedure
  return NewMainProcData->NextProc(hWnd, uMsg, wParam, lParam);
}

LRESULT CALLBACK NewFrameProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
  //Edit parent
  {
   LRESULT lResult;

   if (lResult=EditParentMessages(hWnd, uMsg, wParam, lParam)) return lResult;
  }

  //Call next procedure
  return NewFrameProcData->NextProc(hWnd, uMsg, wParam, lParam);
}

LRESULT CALLBACK EditParentMessages(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
  if (uMsg == WM_NOTIFY)
  {
    if (hWndMainDlg)
    {
      if (wParam == ID_EDIT)
      {
        if (((NMHDR *)lParam)->code == EN_SELCHANGE)
        {
          SendMessage(hWndMainDlg, AKDLL_GETHEX, (WPARAM)((NMHDR *)lParam)->hwndFrom, 0);
        }
      }
    }
  }
  return FALSE;
}

void FillCodepageListboxA(HWND hWnd, int nCodePage)
{
  char szCodepageName[MAX_PATH];
  int *lpCodepageList;
  int nSel=0;
  int nIndex=0;
  int i;

  if (lpCodepageList=(int *)SendMessage(hMainWnd, AKD_GETCODEPAGELIST, 0, 0))
  {
    for (i=0; lpCodepageList[i]; ++i)
    {
      if (lpCodepageList[i] != CP_UNICODE_UCS2_LE && lpCodepageList[i] != CP_UNICODE_UCS2_BE)
      {
        if (nCodePage == lpCodepageList[i]) nSel=nIndex;

        GetCodePageNameA(lpCodepageList[i], szCodepageName, MAX_PATH);
        SendMessageA(hWnd, CB_ADDSTRING, 0, (LPARAM)szCodepageName);
        ++nIndex;
      }
    }
    if (!nSel) nCodePage=lpCodepageList[0];
    SendMessage(hWnd, CB_SETCURSEL, nSel, 0);
  }
}

void FillCodepageListboxW(HWND hWnd, int nCodePage)
{
  wchar_t wszCodepageName[MAX_PATH];
  int *lpCodepageList;
  int nSel=0;
  int nIndex=0;
  int i;

  if (lpCodepageList=(int *)SendMessage(hMainWnd, AKD_GETCODEPAGELIST, 0, 0))
  {
    for (i=0; lpCodepageList[i]; ++i)
    {
      if (lpCodepageList[i] != CP_UNICODE_UCS2_LE && lpCodepageList[i] != CP_UNICODE_UCS2_BE)
      {
        if (nCodePage == lpCodepageList[i]) nSel=nIndex;

        GetCodePageNameW(lpCodepageList[i], wszCodepageName, MAX_PATH);
        SendMessageW(hWnd, CB_ADDSTRING, 0, (LPARAM)wszCodepageName);
        ++nIndex;
      }
    }
    if (!nSel) nCodePage=lpCodepageList[0];
    SendMessage(hWnd, CB_SETCURSEL, nSel, 0);
  }
}

void GetCodePageNameA(int nCodePage, char *szCodePage, int nLen)
{
  CPINFOEXA CPInfoEx;
  char szNumber[32];

  if (nCodePage)
  {
    if (nCodePage == CP_UNICODE_UCS2_LE)
      lstrcpynA(szCodePage, STR_UNICODE_UCS2_LEA, nLen);
    else if (nCodePage == CP_UNICODE_UCS2_BE)
      lstrcpynA(szCodePage, STR_UNICODE_UCS2_BEA, nLen);
    else if (nCodePage == CP_UNICODE_UTF8)
      lstrcpynA(szCodePage, STR_UNICODE_UTF8A, nLen);
    else if (nCodePage == CP_UNICODE_UTF7)
      lstrcpynA(szCodePage, STR_UNICODE_UTF7A, nLen);
    else
    {
      if (GetCPInfoExAPtr && (*GetCPInfoExAPtr)(nCodePage, 0, &CPInfoEx) && nCodePage == xatoiA(CPInfoEx.CodePageName))
      {
        lstrcpynA(szCodePage, CPInfoEx.CodePageName, nLen);
      }
      else
      {
        wsprintfA(szNumber, "%d ", nCodePage);
        lstrcpynA(szCodePage, szNumber, nLen);
      }
    }
  }
  else szCodePage[0]='\0';
}

void GetCodePageNameW(int nCodePage, wchar_t *wszCodePage, int nLen)
{
  CPINFOEXW CPInfoExW;
  wchar_t wszNumber[32];

  if (nCodePage)
  {
    if (nCodePage == CP_UNICODE_UCS2_LE)
      lstrcpynW(wszCodePage, STR_UNICODE_UCS2_LEW, nLen);
    else if (nCodePage == CP_UNICODE_UCS2_BE)
      lstrcpynW(wszCodePage, STR_UNICODE_UCS2_BEW, nLen);
    else if (nCodePage == CP_UNICODE_UTF8)
      lstrcpynW(wszCodePage, STR_UNICODE_UTF8W, nLen);
    else if (nCodePage == CP_UNICODE_UTF7)
      lstrcpynW(wszCodePage, STR_UNICODE_UTF7W, nLen);
    else
    {
      if (GetCPInfoExWPtr && (*GetCPInfoExWPtr)(nCodePage, 0, &CPInfoExW) && nCodePage == xatoiW(CPInfoExW.CodePageName))
      {
        lstrcpynW(wszCodePage, CPInfoExW.CodePageName, nLen);
      }
      else
      {
        wsprintfW(wszNumber, L"%d ", nCodePage);
        lstrcpynW(wszCodePage, wszNumber, nLen);
      }
    }
  }
  else wszCodePage[0]='\0';
}

void GetHex(HWND hWndHexView, HWND hWndEdit)
{
  EDITINFO ei;
  CHARRANGE cr;
  wchar_t *wszInput;
  wchar_t *wszOutput;
  char *szInput;
  char *szOutput;
  int nInputAnsiLen=0;
  int nInputUnicodeLen=0;
  int nOutputAnsiLen=0;
  int nOutputUnicodeLen=0;
  int nCodePage;

  if (SendMessage(hMainWnd, AKD_GETEDITINFO, (WPARAM)NULL, (LPARAM)&ei))
  {
    if (!hWndEdit || ei.hWndEdit == hWndEdit)
    {
      if (bAkelEdit)
      {
         EXGETTEXTRANGE tr;

         SendMessage(ei.hWndEdit, AEM_GETSEL, (WPARAM)NULL, (LPARAM)&tr.cr);
         tr.pText=NULL;
         tr.nNewLine=AELB_ASIS;
         nInputUnicodeLen=SendMessage(hMainWnd, AKD_EXGETTEXTRANGEW, (WPARAM)ei.hWndEdit, (LPARAM)&tr);
         wszInput=(wchar_t *)tr.pText;
      }
      else
      {
        wszInput=(wchar_t *)SendMessage(hMainWnd, AKD_GETSELTEXTW, (WPARAM)ei.hWndEdit, (LPARAM)&nInputUnicodeLen);
        SendMessage(ei.hWndEdit, EM_EXGETSEL, 0, (LPARAM)&cr);
        nInputUnicodeLen=cr.cpMax - cr.cpMin;
      }

      if (nInputUnicodeLen)
      {
        //Use file codepage if not unicode.
        if (nAnsiCP == -1)
        {
          if (ei.nCodePage != CP_UNICODE_UCS2_LE && ei.nCodePage != CP_UNICODE_UCS2_BE)
            nCodePage=ei.nCodePage;
          else
            nCodePage=CP_ACP;
        }
        else nCodePage=nAnsiCP;

        if (nDirection == CT_STR2HEX)
        {
          SetWindowTextA(hWndHexView, "");
          SendMessage(hWndHexView, WM_SETREDRAW, FALSE, 0);

          //Ansi
          nInputAnsiLen=WideCharToMultiByte(nCodePage, 0, wszInput, nInputUnicodeLen, NULL, 0, NULL, NULL);

          if (szInput=(char *)GlobalAlloc(GPTR, nInputAnsiLen + 1))
          {
            WideCharToMultiByte(nCodePage, 0, wszInput, nInputUnicodeLen + 1, szInput, nInputAnsiLen + 1, NULL, NULL);

            if (GetHexFromAnsiStringW(szInput, nInputAnsiLen, wszPrefix, wszSuffix, &wszOutput))
            {
              SendMessage(hMainWnd, AKD_REPLACESELW, (WPARAM)hWndHexView, (LPARAM)wszOutput);
              FreeHexString(wszOutput);
            }
            GlobalFree((HGLOBAL)szInput);
          }
          SendMessage(hMainWnd, AKD_REPLACESELW, (WPARAM)hWndHexView, (LPARAM)L"\n\n");
          SendMessage(hWndHexView, EM_EXGETSEL, 0, (LPARAM)&cr);

          //Unicode
          if (GetHexFromUnicodeStringW(wszInput, nInputUnicodeLen, wszPrefix, wszSuffix, &wszOutput))
          {
            SendMessage(hMainWnd, AKD_REPLACESELW, (WPARAM)hWndHexView, (LPARAM)wszOutput);
            SendMessage(hWndHexView, EM_EXSETSEL, 0, (LPARAM)&cr);
            FreeHexString(wszOutput);
          }

          SendMessage(hWndHexView, WM_SETREDRAW, TRUE, 0);
          InvalidateRect(hWndHexView, NULL, FALSE);
        }
        else if (nDirection == CT_HEX2STR)
        {
          SetWindowTextA(hWndHexView, "");
          SendMessage(hWndHexView, WM_SETREDRAW, FALSE, 0);

          //Ansi
          if (nOutputAnsiLen=GetAnsiStringFromHexW(wszInput, nInputUnicodeLen, wszPrefix, wszSuffix, &szOutput))
          {
            nOutputUnicodeLen=MultiByteToWideChar(nCodePage, 0, szOutput, nOutputAnsiLen, NULL, 0);

            if (wszOutput=(wchar_t *)GlobalAlloc(GPTR, nOutputUnicodeLen * sizeof(wchar_t) + 2))
            {
              MultiByteToWideChar(nCodePage, 0, szOutput, nOutputAnsiLen + 1, wszOutput, nOutputUnicodeLen + 1);

              SendMessage(hMainWnd, AKD_REPLACESELW, (WPARAM)hWndHexView, (LPARAM)wszOutput);

              GlobalFree((HGLOBAL)wszOutput);
            }
            FreeHexString(szOutput);
          }
          SendMessage(hMainWnd, AKD_REPLACESELW, (WPARAM)hWndHexView, (LPARAM)L"\n\n");
          SendMessage(hWndHexView, EM_EXGETSEL, 0, (LPARAM)&cr);

          //Unicode
          if (GetUnicodeStringFromHexW(wszInput, nInputUnicodeLen, wszPrefix, wszSuffix, &wszOutput))
          {
            SendMessage(hMainWnd, AKD_REPLACESELW, (WPARAM)hWndHexView, (LPARAM)wszOutput);
            SendMessage(hWndHexView, EM_EXSETSEL, 0, (LPARAM)&cr);
            FreeHexString(wszOutput);
          }
          SendMessage(hWndHexView, WM_SETREDRAW, TRUE, 0);
          InvalidateRect(hWndHexView, NULL, FALSE);
        }

        SendMessage(hMainWnd, AKD_FREETEXT, 0, (LPARAM)wszInput);
      }
      else
      {
        SetWindowTextA(hWndHexView, "");
      }
    }
  }
  else SetWindowTextA(hWndHexView, "");
}

int GetHexFromAnsiStringW(char *szInput, int nInputLen, wchar_t *wszPrefix, wchar_t *wszSuffix, wchar_t **wppOutput)
{
  wchar_t *wszOutput;
  int nPrefixLen;
  int nSuffixLen;
  int nSymbolLen;
  int nOutputLen;
  int a=0;
  int b=0;

  nPrefixLen=lstrlenW(wszPrefix);
  nSuffixLen=lstrlenW(wszSuffix);
  nSymbolLen=nPrefixLen + 2 + nSuffixLen;
  nOutputLen=nInputLen * nSymbolLen;

  if (wszOutput=*wppOutput=(wchar_t *)GlobalAlloc(GPTR, (nOutputLen + 1) * sizeof(wchar_t)))
  {
    for (; a < nInputLen && (b + nSymbolLen) <= nOutputLen; ++a)
    {
      memcpy(wszOutput + b, wszPrefix, nPrefixLen * sizeof(wchar_t));
      b+=nPrefixLen;

      b+=dec2hexW((unsigned char)szInput[a], wszOutput + b, 2, FALSE);

      memcpy(wszOutput + b, wszSuffix, nSuffixLen * sizeof(wchar_t));
      b+=nSuffixLen;
    }
    *(wszOutput + b)='\0';
  }

  if (!b)
  {
    GlobalFree(*wppOutput);
    *wppOutput=NULL;
  }
  return b;
}

int GetHexFromUnicodeStringW(wchar_t *wszInput, int nInputLen, wchar_t *wszPrefix, wchar_t *wszSuffix, wchar_t **wppOutput)
{
  wchar_t *wszOutput;
  wchar_t wchChar='\0';
  int nPrefixLen;
  int nSuffixLen;
  int nSymbolLen;
  int nOutputLen;
  int a=0;
  int b=0;

  nPrefixLen=lstrlenW(wszPrefix);
  nSuffixLen=lstrlenW(wszSuffix);
  nSymbolLen=nPrefixLen + 4 + nSuffixLen;
  nOutputLen=nInputLen * nSymbolLen;

  if (wszOutput=*wppOutput=(wchar_t *)GlobalAlloc(GPTR, (nOutputLen + 1) * sizeof(wchar_t)))
  {
    for (; a < nInputLen && (b + nSymbolLen) <= nOutputLen; ++a)
    {
      memcpy(wszOutput + b, wszPrefix, nPrefixLen * sizeof(wchar_t));
      b+=nPrefixLen;

      if (nUnicodeCP == CP_UNICODE_UCS2_LE)
        wchChar=wszInput[a];
      else if (nUnicodeCP == CP_UNICODE_UCS2_BE)
        wchChar=MAKEWORD(HIBYTE(wszInput[a]), LOBYTE(wszInput[a]));
      b+=dec2hexW(wchChar, wszOutput + b, 4, FALSE);

      memcpy(wszOutput + b, wszSuffix, nSuffixLen * sizeof(wchar_t));
      b+=nSuffixLen;
    }
    *(wszOutput + b)='\0';
  }

  if (!b)
  {
    GlobalFree(*wppOutput);
    *wppOutput=NULL;
  }
  return b;
}

int GetAnsiStringFromHexW(wchar_t *wszInput, int nInputLen, wchar_t *wszPrefix, wchar_t *wszSuffix, char **ppOutput)
{
  wchar_t wszHexChar[3];
  char *szOutput;
  int nPrefixLen;
  int nSuffixLen;
  int nSymbolLen;
  int nOutputLen;
  int nHexChar;
  int a=0;
  int b=0;

  nPrefixLen=lstrlenW(wszPrefix);
  nSuffixLen=lstrlenW(wszSuffix);
  nSymbolLen=nPrefixLen + 2 + nSuffixLen;
  nOutputLen=nInputLen / nSymbolLen + nSymbolLen;

  if (szOutput=*ppOutput=(char *)GlobalAlloc(GPTR, nOutputLen + 1))
  {
    while (a < nInputLen)
    {
      if (!memcmp(wszInput + a, wszPrefix, nPrefixLen * sizeof(wchar_t)))
      {
        a+=nPrefixLen;

        if (!wszInput[a]) break;
        wszHexChar[0]=wszInput[a++];
        if (!wszInput[a]) break;
        wszHexChar[1]=wszInput[a++];
        wszHexChar[2]='\0';

        if (!memcmp(wszInput + a, wszSuffix, nSuffixLen * sizeof(wchar_t)))
        {
          a+=nSuffixLen;
          nHexChar=hex2decW(wszHexChar);

          if (nHexChar > 0)
            szOutput[b++]=nHexChar;
          else if (nHexChar == 0)
            szOutput[b++]=' ';
          else
            continue;
        }
      }
      else ++a;
    }
    szOutput[b]='\0';
  }

  if (!b)
  {
    GlobalFree(*ppOutput);
    *ppOutput=NULL;
  }
  return b;
}

int GetUnicodeStringFromHexW(wchar_t *wszInput, int nInputLen, wchar_t *wszPrefix, wchar_t *wszSuffix, wchar_t **wppOutput)
{
  wchar_t wszHexChar[5];
  wchar_t *wszOutput;
  int nPrefixLen;
  int nSuffixLen;
  int nSymbolLen;
  int nOutputLen;
  int nHexChar;
  int a=0;
  int b=0;

  nPrefixLen=lstrlenW(wszPrefix);
  nSuffixLen=lstrlenW(wszSuffix);
  nSymbolLen=nPrefixLen + 4 + nSuffixLen;
  nOutputLen=nInputLen / nSymbolLen + nSymbolLen;

  if (wszOutput=*wppOutput=(wchar_t *)GlobalAlloc(GPTR, (nOutputLen + 1) * sizeof(wchar_t)))
  {
    while (a < nInputLen)
    {
      if (!memcmp(wszInput + a, wszPrefix, nPrefixLen * sizeof(wchar_t)))
      {
        a+=nPrefixLen;

        if (nUnicodeCP == CP_UNICODE_UCS2_LE)
        {
          if (!wszInput[a]) break;
          wszHexChar[0]=wszInput[a++];
          if (!wszInput[a]) break;
          wszHexChar[1]=wszInput[a++];
          if (!wszInput[a]) break;
          wszHexChar[2]=wszInput[a++];
          if (!wszInput[a]) break;
          wszHexChar[3]=wszInput[a++];
        }
        else if (nUnicodeCP == CP_UNICODE_UCS2_BE)
        {
          if (!wszInput[a]) break;
          wszHexChar[2]=wszInput[a++];
          if (!wszInput[a]) break;
          wszHexChar[3]=wszInput[a++];
          if (!wszInput[a]) break;
          wszHexChar[0]=wszInput[a++];
          if (!wszInput[a]) break;
          wszHexChar[1]=wszInput[a++];
        }
        wszHexChar[4]='\0';

        if (!memcmp(wszInput + a, wszSuffix, nSuffixLen * sizeof(wchar_t)))
        {
          a+=nSuffixLen;
          nHexChar=hex2decW(wszHexChar);

          if (nHexChar > 0)
            wszOutput[b++]=nHexChar;
          else if (nHexChar == 0)
            wszOutput[b++]=' ';
          else
            continue;
        }
      }
      else ++a;
    }
    wszOutput[b]='\0';
  }

  if (!b)
  {
    GlobalFree(*wppOutput);
    *wppOutput=NULL;
  }
  return b;
}

void FreeHexString(void *lpString)
{
  GlobalFree((HGLOBAL)lpString);
}

void UpdateSelection()
{
  EDITINFO ei;
  NMHDR nmhdr;

  if (SendMessage(hMainWnd, AKD_GETEDITINFO, (WPARAM)NULL, (LPARAM)&ei))
  {
    nmhdr.hwndFrom=ei.hWndEdit;
    nmhdr.idFrom=ID_EDIT;
    nmhdr.code=EN_SELCHANGE;
    SendMessage(GetParent(ei.hWndEdit), WM_NOTIFY, ID_EDIT, (LPARAM)&nmhdr);
  }
}


//// Options

void ReadOptionsA()
{
  PLUGINOPTIONA po;
  HANDLE hOptions;

  if (hOptions=(HANDLE)SendMessage(hMainWnd, AKD_BEGINOPTIONS, POB_READ, (LPARAM)szPluginName))
  {
    po.pOptionName="DockSide";
    po.dwType=PO_DWORD;
    po.lpData=(LPBYTE)&nDockSide;
    po.dwData=sizeof(DWORD);
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    po.pOptionName="DockRect";
    po.dwType=PO_BINARY;
    po.lpData=(LPBYTE)&rcDockRect;
    po.dwData=sizeof(RECT);
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    po.pOptionName="Direction";
    po.dwType=PO_DWORD;
    po.lpData=(LPBYTE)&nDirection;
    po.dwData=sizeof(DWORD);
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    po.pOptionName="Prefix";
    po.dwType=PO_STRING;
    po.lpData=(LPBYTE)szPrefix;
    po.dwData=MAX_PATH;
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    po.pOptionName="Suffix";
    po.dwType=PO_STRING;
    po.lpData=(LPBYTE)szSuffix;
    po.dwData=MAX_PATH;
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    po.pOptionName="Ansi";
    po.dwType=PO_DWORD;
    po.lpData=(LPBYTE)&nAnsiCP;
    po.dwData=sizeof(DWORD);
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    po.pOptionName="Unicode";
    po.dwType=PO_DWORD;
    po.lpData=(LPBYTE)&nUnicodeCP;
    po.dwData=sizeof(DWORD);
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    SendMessage(hMainWnd, AKD_ENDOPTIONS, (WPARAM)hOptions, 0);
  }
}

void ReadOptionsW()
{
  PLUGINOPTIONW po;
  HANDLE hOptions;

  if (hOptions=(HANDLE)SendMessage(hMainWnd, AKD_BEGINOPTIONS, POB_READ, (LPARAM)wszPluginName))
  {
    po.pOptionName=L"DockSide";
    po.dwType=PO_DWORD;
    po.lpData=(LPBYTE)&nDockSide;
    po.dwData=sizeof(DWORD);
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    po.pOptionName=L"DockRect";
    po.dwType=PO_BINARY;
    po.lpData=(LPBYTE)&rcDockRect;
    po.dwData=sizeof(RECT);
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    po.pOptionName=L"Direction";
    po.dwType=PO_DWORD;
    po.lpData=(LPBYTE)&nDirection;
    po.dwData=sizeof(DWORD);
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    po.pOptionName=L"Prefix";
    po.dwType=PO_STRING;
    po.lpData=(LPBYTE)wszPrefix;
    po.dwData=MAX_PATH * sizeof(wchar_t);
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    po.pOptionName=L"Suffix";
    po.dwType=PO_STRING;
    po.lpData=(LPBYTE)wszSuffix;
    po.dwData=MAX_PATH * sizeof(wchar_t);
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    po.pOptionName=L"Ansi";
    po.dwType=PO_DWORD;
    po.lpData=(LPBYTE)&nAnsiCP;
    po.dwData=sizeof(DWORD);
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    po.pOptionName=L"Unicode";
    po.dwType=PO_DWORD;
    po.lpData=(LPBYTE)&nUnicodeCP;
    po.dwData=sizeof(DWORD);
    SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

    SendMessage(hMainWnd, AKD_ENDOPTIONS, (WPARAM)hOptions, 0);
  }
}

void SaveOptionsA(DWORD dwFlags)
{
  PLUGINOPTIONA po;
  HANDLE hOptions;

  if (hOptions=(HANDLE)SendMessage(hMainWnd, AKD_BEGINOPTIONS, POB_SAVE, (LPARAM)szPluginName))
  {
    if (dwFlags & OF_WINDOWRECT)
    {
      po.pOptionName="DockSide";
      po.dwType=PO_DWORD;
      po.lpData=(LPBYTE)&dkMainDlg->nSide;
      po.dwData=sizeof(DWORD);
      SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

      po.pOptionName="DockRect";
      po.dwType=PO_BINARY;
      po.lpData=(LPBYTE)&dkMainDlg->rcSize;
      po.dwData=sizeof(RECT);
      SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);
    }

    if (dwFlags & OF_SETTINGS)
    {
      po.pOptionName="Direction";
      po.dwType=PO_DWORD;
      po.lpData=(LPBYTE)&nDirection;
      po.dwData=sizeof(DWORD);
      SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

      po.pOptionName="Prefix";
      po.dwType=PO_STRING;
      po.lpData=(LPBYTE)szPrefix;
      po.dwData=lstrlenA(szPrefix) + 1;
      SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

      po.pOptionName="Suffix";
      po.dwType=PO_STRING;
      po.lpData=(LPBYTE)szSuffix;
      po.dwData=lstrlenA(szSuffix) + 1;
      SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

      po.pOptionName="Ansi";
      po.dwType=PO_DWORD;
      po.lpData=(LPBYTE)&nAnsiCP;
      po.dwData=sizeof(DWORD);
      SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

      po.pOptionName="Unicode";
      po.dwType=PO_DWORD;
      po.lpData=(LPBYTE)&nUnicodeCP;
      po.dwData=sizeof(DWORD);
      SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);
    }

    SendMessage(hMainWnd, AKD_ENDOPTIONS, (WPARAM)hOptions, 0);
  }
}

void SaveOptionsW(DWORD dwFlags)
{
  PLUGINOPTIONW po;
  HANDLE hOptions;

  if (hOptions=(HANDLE)SendMessage(hMainWnd, AKD_BEGINOPTIONS, POB_SAVE, (LPARAM)wszPluginName))
  {
    if (dwFlags & OF_WINDOWRECT)
    {
      po.pOptionName=L"DockSide";
      po.dwType=PO_DWORD;
      po.lpData=(LPBYTE)&dkMainDlg->nSide;
      po.dwData=sizeof(DWORD);
      SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

      po.pOptionName=L"DockRect";
      po.dwType=PO_BINARY;
      po.lpData=(LPBYTE)&dkMainDlg->rcSize;
      po.dwData=sizeof(RECT);
      SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);
    }

    if (dwFlags & OF_SETTINGS)
    {
      po.pOptionName=L"Direction";
      po.dwType=PO_DWORD;
      po.lpData=(LPBYTE)&nDirection;
      po.dwData=sizeof(DWORD);
      SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

      po.pOptionName=L"Prefix";
      po.dwType=PO_STRING;
      po.lpData=(LPBYTE)wszPrefix;
      po.dwData=lstrlenW(wszPrefix) * sizeof(wchar_t) + 2;
      SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

      po.pOptionName=L"Suffix";
      po.dwType=PO_STRING;
      po.lpData=(LPBYTE)wszSuffix;
      po.dwData=lstrlenW(wszSuffix) * sizeof(wchar_t) + 2;
      SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

      po.pOptionName=L"Ansi";
      po.dwType=PO_DWORD;
      po.lpData=(LPBYTE)&nAnsiCP;
      po.dwData=sizeof(DWORD);
      SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);

      po.pOptionName=L"Unicode";
      po.dwType=PO_DWORD;
      po.lpData=(LPBYTE)&nUnicodeCP;
      po.dwData=sizeof(DWORD);
      SendMessage(hMainWnd, AKD_OPTION, (WPARAM)hOptions, (LPARAM)&po);
    }

    SendMessage(hMainWnd, AKD_ENDOPTIONS, (WPARAM)hOptions, 0);
  }
}

char* GetLangStringA(LANGID wLangID, int nStringID)
{
  static char szStringBuf[MAX_PATH];

  WideCharToMultiByte(CP_ACP, 0, GetLangStringW(wLangID, nStringID), -1, szStringBuf, MAX_PATH, NULL, NULL);
  return szStringBuf;
}

wchar_t* GetLangStringW(LANGID wLangID, int nStringID)
{
  if (wLangID == LANG_RUSSIAN)
  {
    if (nStringID == STRID_DIRECTION)
      return L"\x041D\x0430\x043F\x0440\x0430\x0432\x043B\x0435\x043D\x0438\x0435";
    if (nStringID == STRID_STR2HEX)
      return L"\x0422\x0435\x043A\x0441\x0442\x0020\x002D\x003E\x0020\x0048\x0065\x0078";
    if (nStringID == STRID_HEX2STR)
      return L"\x0048\x0065\x0078\x0020\x002D\x003E\x0020\x0422\x0435\x043A\x0441\x0442";
    if (nStringID == STRID_SETUP)
      return L"\x041D\x0430\x0441\x0442\x0440\x043E\x0439\x043A\x0438\x002E\x002E\x002E";
    if (nStringID == STRID_ENCLOSE)
      return L"\x041E\x0431\x0440\x0430\x043C\x043B\x0435\x043D\x0438\x0435";
    if (nStringID == STRID_PREFIX)
      return L"\x041F\x0440\x0435\x0444\x0438\x043A\x0441\x003A";
    if (nStringID == STRID_SUFFIX)
      return L"\x0421\x0443\x0444\x0444\x0438\x043A\x0441\x003A";
    if (nStringID == STRID_SINGLEBYTE)
      return L"\x041E\x0434\x0438\x043D\x0020\x0431\x0430\x0439\x0442";
    if (nStringID == STRID_AUTO)
      return L"\x0410\x0432\x0442\x043E\x043C\x0430\x0442\x0438\x0447\x0435\x0441\x043A\x0438";
    if (nStringID == STRID_DOUBLEBYTE)
      return L"\x0414\x0432\x0430\x0020\x0431\x0430\x0439\x0442\x0430";
    if (nStringID == STRID_OK)
      return L"\x004F\x004B";
    if (nStringID == STRID_CANCEL)
      return L"\x041E\x0442\x043C\x0435\x043D\x0430";
  }
  else
  {
    if (nStringID == STRID_DIRECTION)
      return L"Direction";
    if (nStringID == STRID_STR2HEX)
      return L"Text -> Hex";
    if (nStringID == STRID_HEX2STR)
      return L"Hex -> Text";
    if (nStringID == STRID_SETUP)
      return L"Settings...";
    if (nStringID == STRID_ENCLOSE)
      return L"Enclose";
    if (nStringID == STRID_PREFIX)
      return L"Prefix:";
    if (nStringID == STRID_SUFFIX)
      return L"Suffix:";
    if (nStringID == STRID_SINGLEBYTE)
      return L"Single byte";
    if (nStringID == STRID_AUTO)
      return L"Auto";
    if (nStringID == STRID_DOUBLEBYTE)
      return L"Double byte";
    if (nStringID == STRID_OK)
      return L"OK";
    if (nStringID == STRID_CANCEL)
      return L"Cancel";
  }
  return L"";
}

void InitCommon(PLUGINDATA *pd)
{
  bInitCommon=TRUE;
  hInstanceDLL=pd->hInstanceDLL;
  bOnMainStart=pd->bOnStart;
  hMainWnd=pd->hMainWnd;
  hWndEdit=pd->hWndEdit;
  hMainIcon=pd->hMainIcon;
  bOldWindows=pd->bOldWindows;
  bAkelEdit=pd->bAkelEdit;
  bMDI=pd->bMDI;
  wLangSystem=PRIMARYLANGID(pd->wLangSystem);

  if (bOldWindows)
  {
    int i;

    for (i=0; ((char *)pd->pFunction)[i] != ':'; ++i)
      szPluginName[i]=((char *)pd->pFunction)[i];
    szPluginName[i]='\0';
  }
  else
  {
    int i;

    for (i=0; ((wchar_t *)pd->pFunction)[i] != ':'; ++i)
      wszPluginName[i]=((wchar_t *)pd->pFunction)[i];
    wszPluginName[i]='\0';
  }

  //Read settings
  if (bOldWindows)
  {
    HMODULE hKernel32;

    hKernel32=GetModuleHandleA("kernel32.dll");
    GetCPInfoExAPtr=(BOOL (WINAPI *)(UINT, DWORD, LPCPINFOEXA))GetProcAddress(hKernel32, "GetCPInfoExA");
    ReadOptionsA();
    MultiByteToWideChar(CP_ACP, 0, szPrefix, -1, wszPrefix, MAX_PATH);
    MultiByteToWideChar(CP_ACP, 0, szSuffix, -1, wszSuffix, MAX_PATH);
  }
  else
  {
    HMODULE hKernel32;

    hKernel32=GetModuleHandleW(L"kernel32.dll");
    GetCPInfoExWPtr=(BOOL (WINAPI *)(UINT, DWORD, LPCPINFOEXW))GetProcAddress(hKernel32, "GetCPInfoExW");
    ReadOptionsW();
  }

  //Context menu
  hPopupView=GetSubMenu(pd->hPopupMenu, 1);
}

void InitMain()
{
  bInitMain=TRUE;

  //SubClass
  NewMainProcData=NULL;
  SendMessage(hMainWnd, AKD_SETMAINPROC, (WPARAM)NewMainProc, (LPARAM)&NewMainProcData);

  if (bMDI)
  {
    NewFrameProcData=NULL;
    SendMessage(hMainWnd, AKD_SETFRAMEPROC, (WPARAM)NewFrameProc, (LPARAM)&NewFrameProcData);
  }
}

void UninitMain()
{
  bInitMain=FALSE;

  //Remove subclass
  if (NewMainProcData)
  {
    SendMessage(hMainWnd, AKD_SETMAINPROC, (WPARAM)NULL, (LPARAM)&NewMainProcData);
    NewMainProcData=NULL;
  }
  if (bMDI)
  {
    if (NewFrameProcData)
    {
      SendMessage(hMainWnd, AKD_SETFRAMEPROC, (WPARAM)NULL, (LPARAM)&NewFrameProcData);
      NewFrameProcData=NULL;
    }
  }
}

//Entry point
extern "C" BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
{
  if (fdwReason == DLL_PROCESS_ATTACH)
  {
  }
  else if (fdwReason == DLL_THREAD_ATTACH)
  {
  }
  else if (fdwReason == DLL_THREAD_DETACH)
  {
  }
  else if (fdwReason == DLL_PROCESS_DETACH)
  {
  }
  return TRUE;
}
